@@ -26,10 +26,10 @@
 #include <gtk/gtk.h>
 
 #include "ev-document-fonts.h"
-#include "ev-page-cache.h"
 #include "ev-properties-dialog.h"
 #include "ev-properties-fonts.h"
 #include "ev-properties-view.h"
+#include "ev-properties-license.h"
 
 struct _EvPropertiesDialog {
 	GtkDialog base_instance;
@@ -38,6 +38,7 @@ struct _EvPropertiesDialog {
 	GtkWidget *notebook;
 	GtkWidget *general_page;
 	GtkWidget *fonts_page;
+	GtkWidget *license_page;
 };
 
 struct _EvPropertiesDialogClass {
@@ -77,6 +78,7 @@ ev_properties_dialog_init (EvPropertiesDialog *properties)
 
 void
 ev_properties_dialog_set_document (EvPropertiesDialog *properties,
+				   const gchar        *uri,
 			           EvDocument         *document)
 {
 	GtkWidget *label;
@@ -84,11 +86,11 @@ ev_properties_dialog_set_document (EvPropertiesDialog *properties,
 
 	properties->document = document;
 
-	info = ev_page_cache_get_info (ev_page_cache_get (document));
+	info = ev_document_get_info (document);
 
 	if (properties->general_page == NULL) {
 		label = gtk_label_new (_("General"));
-		properties->general_page = ev_properties_view_new ();
+		properties->general_page = ev_properties_view_new (uri);
 		gtk_notebook_append_page (GTK_NOTEBOOK (properties->notebook),
 					  properties->general_page, label);
 		gtk_widget_show (properties->general_page);
@@ -107,6 +109,19 @@ ev_properties_dialog_set_document (EvPropertiesDialog *properties,
 		ev_properties_fonts_set_document
 			(EV_PROPERTIES_FONTS (properties->fonts_page), document);
 	}
+
+	if (info->fields_mask & EV_DOCUMENT_INFO_LICENSE && info->license) {
+		if (properties->license_page == NULL) {
+			label = gtk_label_new (_("Document License"));
+			properties->license_page = ev_properties_license_new ();
+			gtk_notebook_append_page (GTK_NOTEBOOK (properties->notebook),
+						  properties->license_page, label);
+			gtk_widget_show (properties->license_page);
+		}
+
+		ev_properties_license_set_license
+			(EV_PROPERTIES_LICENSE (properties->license_page), info->license);
+	}
 }
 
 GtkWidget *
